#include "mylib.h"
#include "StarterRsc.h"

//#include <list.h>

extern SPreferenceType FidoReaderPrefs;
extern MemHandle fonth;
CFidoDBList::CFidoDBList()
{
	status=0;
	for(int i=0;i<FIDO_MAX_AREA_COUNT;i++)
	{
		name[i]=NULL;
		dbid[i]=0;
	};
	area_count=0;
};

CFidoDBList::~CFidoDBList()
{
	Free();
};

void CFidoDBList::Free()
{
	if(status)
	{
		for(int i=0;i<area_count;i++)
		{
			if(name[i])
			{
				delete[]name[i];
				name[i]=NULL;
			};
			dbid[i]=0;
		};
		status=0;
	};
};

void CFidoDBList::Init()
{
	if(status)
		Free();

	area_count=mlGetDBIDList(dbid,FIDO_MAX_AREA_COUNT,"fddb","GaUs");
	
	if(area_count)
	{
		int i;
		UInt16 tsize;
		SAreaHeader sah;
		DmOpenRef dbP;
		
		char buf[32];
		char tbuf[64],tbuflen;

		all_area_size=0;
		
		for(i=0;i<area_count;i++)
		{
			MemSet(buf,32,0);
			if(DmDatabaseInfo(0,dbid[i],buf,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL)!=errNone)
			{
				FrmAlert(SomeErrorAlert);
				Free();
				return;
			};
			
			if(StrNCompare(buf,"fd",2)!=0)
			{
				FrmAlert(IncorrectNameAlert);
				Free();
				return;
			};
		
			if((dbP=DmOpenDatabase(0,dbid[i],dmModeReadOnly))!=NULL)
			{
				if(DmNumRecords(dbP)==0)
				{
					FrmCustomAlert(FidoBaseCorruptAlert,buf,NULL,NULL);
					Free();
					return;
				};
				
				if(mlCopyRecordToMem(dbP,0,&sah,0,32))
				{
					if(sah.version!=0x2801)
					{
						FrmAlert(SomeErrorAlert);
						Free();
						return;
					};
					
					tsize=(UInt16)(mlGetDBSize(0,dbid[i])/1024);
					all_area_size+=tsize;
					
//					StrToLower(buf+2,buf+2);tbuflen=StrPrintF(tbuf,"%s(%dk)|%d|%d|",buf+2,tsize,sah.unread,sah.count);
					tbuflen=StrPrintF(tbuf,"%s|%d|%d|",buf+2,sah.unread,sah.count);
					
					if(tbuflen<0)
					{
						FrmAlert(SomeErrorAlert);
						Free();
						return;
					};
					
					name[i]=new char [tbuflen+1];
					
					if(!name[i])
					{
						FrmAlert(SomeErrorAlert);
						Free();
						return;
					};
					
					MemSet(name[i],tbuflen+1,0);
					StrCopy(name[i],tbuf);

				}
				else
				{
					FrmCustomAlert(FidoBaseCorruptAlert,buf,NULL,NULL);
					Free();
					return;
				};
				
				DmCloseDatabase(dbP);
			}
			else
			{
				FrmCustomAlert(DBOpenErrorAlert,buf,NULL,NULL);
				Free();
				return;

			};
		};
		status=1;
		Sort();
		return;
	}
//	else
//		FrmAlert(NoFidoDataBasesAlert);
};

LocalID CFidoDBList::GetLocalID(UInt16 index)
{
	if(status)
		if(index<area_count)
			return dbid[index];
			
	return 0;
};

void CFidoDBList::Swap(int i1,int i2)
{
	char* pchar;
	LocalID tint;
	
	pchar=name[i1];
	name[i1]=name[i2];
	name[i2]=pchar;
	
	tint=dbid[i1];
	dbid[i1]=dbid[i2];
	dbid[i2]=tint;
};

void CFidoDBList::Sort()
{
	if(status)
	{
		int i,j,p;
	
		for(i=0;i<area_count-1;i++)
		{
			p=i;

			for(j=i+1;j<area_count;j++)
				if(StrCompare(name[p],name[j])>0)
					p=j;
					
			Swap(i,p);
		};
	};
};

LocalID CFidoDBList::SelectDB()
{
	if(status)
	{
		FormType *frmp;
		ListPtr listp;
		int selected_index;
		
		frmp=FrmGetActiveForm();

		if(area_count<10)
			LstNewList((void**)&frmp,9001,5,20,149,area_count*11,stdFont,area_count,0);
		else
			LstNewList((void**)&frmp,9001,5,20,149,110,stdFont,10,0);
		
		listp=(ListPtr)FrmGetObjectPtr(frmp,FrmGetObjectIndex(frmp,9001));
		LstSetListChoices (listp,name,area_count);
		LstSetDrawFunction (listp,AreaPopUpListDraw2);
		selected_index=LstPopupList(listp);
		FrmRemoveObject(&frmp,FrmGetObjectIndex(frmp,9001));

		if(selected_index==-1)
			return 0;
		else
			return dbid[selected_index];
	}
	else
		return 0;
};

void CFidoDBList::ToList(ListType* lstP)
{
	if(status)
	{
		LstEraseList(lstP);

		LstSetDrawFunction(lstP,AreaPopUpListDraw);
		LstSetListChoices(lstP,name,area_count);
		
/*		if(selected_item_index<area_count)
		{
			LstSetSelection(lstP,selected_item_index);
			
			if(area_count>12)
			{
				if(area_count-selected_item_index<12)
					LstSetTopItem(lstP,area_count-12);
				else
					LstSetTopItem(lstP,selected_item_index);
			};
		}*/
//		else
		LstSetSelection(lstP,-1);
		LstDrawList(lstP);
	}
	else
	{
		LstEraseList(lstP);
		LstSetDrawFunction(lstP,NULL);
		LstSetListChoices(lstP,NULL,0);
		LstDrawList(lstP);
	};
};

CArea::CArea()
{
	status=0;
};

CArea::~CArea()
{
	Free();
};

void CArea::Free()
{
	if(status)
	{
		status=0;

		if(dbP)
		{
			DmCloseDatabase(dbP);
			dbP=0;
		};
			
	};
};

LocalID CArea::Create(char *area_name)
{
	if(status)
		return 0;
		
	char buf[40];
	
	StrCopy(buf,"fd");
	StrCat(buf,area_name);
	
	if(DmCreateDatabase(0,buf,*(UInt32*)"GaUs",*(UInt32*)"fddb",false)==errNone)
	{
		SAreaHeader sah;
		LocalID dbid;
	
		MemSet((void*)&sah,32,0);

		sah.version=0x2801;
		sah.count=0;
		sah.unread=0;
		sah.current_index=0;
		
		dbid=DmFindDatabase(0,buf);
		dbP=DmOpenDatabase(0,dbid,dmModeReadWrite);
		
		if(!mlAddRecord(dbP,(void*)&sah,sizeof(SAreaHeader)))
		{
			FrmCustomAlert(CreateErrorAlert,buf,NULL,NULL);
			return 0;
		};
		
		count=0;
		unread=0;
		current_index=0;
		
		status=1;
		
		return dbid;
	}
	else
	{
		FrmCustomAlert(CreateErrorAlert,buf,NULL,NULL);
		return 0;
	};
	return 0;
};

char CArea::Open(LocalID dbid)
{
	if(status)
		return 0;
		
	SAreaHeader sah;
	UInt16 i;
	MemHandle memh;
	SMessage* pMessage;
	
	dbP=DmOpenDatabase(0,dbid,dmModeReadWrite);
	
	if(!mlCopyRecordToMem(dbP,0,(void*)&sah,0,8))
	{
		FrmCustomAlert(OpenErrorAlert,"   ",NULL,NULL);
		DmCloseDatabase(dbP);
		dbP=0;
		return 0;
	};
	
	if(sah.version!=0x2801)
	{
		FrmCustomAlert(OpenErrorAlert,"  ",NULL,NULL);
		DmCloseDatabase(dbP);
		dbP=0;
		return 0;
	}
	
	if(DmNumRecords(dbP)!=sah.count+1)
	{
		FrmCustomAlert(OpenErrorAlert," . ( ,  )",NULL,NULL);
		sah.count=DmNumRecords(dbP)-1;
		sah.unread=0;
		sah.current_index=0;
	}
	
	count=sah.count;
	unread=0;
//	unread=sah.unread;
	current_index=sah.current_index;

	for(i=0;i<count;i++)
	{
		memh=DmGetRecord(dbP,i+1);
		
		if(!memh)
		{
			FrmAlert(SomeErrorAlert);
			DmCloseDatabase(dbP);
			dbP=0;
			status=0;
			return 0;
		}
	
		pMessage=(SMessage*)MemHandleLock(memh);
		
		msgid[i]=pMessage->msgid;
		reply[i]=pMessage->reply;
		
		if((pMessage->attributes&0x0001)==0)
			unread++;

		MemHandleUnlock(memh);
		DmReleaseRecord(dbP,i+1,true);
	};
	status=1;
	return 1;
};

char CArea::Close()
{
	if(!status)
		return 0;
		
	SAreaHeader sah;
	
	sah.version=0x2801;
	sah.count=count;
	sah.unread=unread;
	sah.current_index=current_index;
	
	if(!mlWriteToRecord(dbP,0,(void*)&sah,0,8))
		FrmAlert(CloseErrorAlert);

	Free();
	return 1;
};

char CArea::AddMessage(Spkt* pkt)
{
	if(!status)
		return 0;
		
	SMessage msg;
	
	msg.msgid=pkt->msgid;				//4		0
	msg.reply=pkt->reply;				//4		4
	msg.attributes=0;						//2		8
	msg.datetime=mlGetSecondsByStrData(pkt->data);						//4		10

	msg.arealen=StrLen(pkt->area)+1;	//1		14
	msg.fromlen=StrLen(pkt->from)+1;	//1		15
	msg.tolen=StrLen(pkt->to)+1;		//1		16
	msg.addrlen=StrLen(pkt->addr)+1;	//1		17
	msg.subjlen=StrLen(pkt->subj)+1;	//1		18
	msg.textlen=StrLen(pkt->text)+1;	//2		19

	msg.area=pkt->area;					//-		21
	msg.from=pkt->from;
	msg.to=pkt->to;
	msg.addr=pkt->addr;
	msg.subj=pkt->subj;
	msg.text=pkt->text;
	
	return AddMessage(&msg);

};

UInt16 CArea::GetMessageAttributes(int index)
{
	if(!status)
		return 0xFFFF;
		
	if(index>=0 && index<count)
	{		
		UInt16 t;
		if(mlCopyRecordToMem(dbP,index+1,(void*)&t,8,2))
			return t;
	}
	return 0xFFFF;
};

void CArea::SetMessageAttributes(int index,UInt16 attr)
{
	if(index>=0 && index<count)
	{
		mlWriteToRecord(dbP,index+1,(void*)&attr,8,2);
	};
};

char CArea::GetMessageAttribute(int index,UInt16 msg_attr)
{
	return (GetMessageAttributes(index)&msg_attr)?1:0;
};

void CArea::SetMessageAttribute(int index,char value,UInt16 msg_attr)
{
	UInt16 tattr;

	tattr=GetMessageAttributes(index);
	
	if(tattr!=0xFFFF)
	{
		if(value)
		{
			if(!(tattr&msg_attr))
			{
				if(msg_attr==FIDO_MSG_ATTR_READ)
					unread--;
				tattr=tattr|msg_attr;
			}
		}
		else
		{
			if(tattr&msg_attr)
			{
				if(msg_attr==FIDO_MSG_ATTR_READ)
					unread++;
				tattr=tattr&(!msg_attr);
			}
		}
		SetMessageAttributes(index,tattr);
	};
};

char CArea::DeleteMessageByIndex(UInt16 index)
{
	if(!status)
		return 0;
		
	if(index>=0 && index<count)
	{
		UInt16 i;

		if(!GetMessageAttribute(index,FIDO_MSG_ATTR_READ))
			unread--;
		
		if(index<current_index)
			current_index--;
		
		count--;

		if(current_index==count && count)
			current_index--;
	
		
		for(i=index;i<count;i++)
		{
			msgid[i]=msgid[i+1];
			reply[i]=reply[i+1];
		};
		
		DmRemoveRecord(dbP,index+1);
		return 1;
	}
	return 0;
};

UInt16 CArea::GetMessageLen(SMessage* msg)
{
	return 22+msg->arealen+msg->fromlen+msg->tolen+msg->addrlen+msg->subjlen+msg->textlen;
};

char CArea::AddMessage(SMessage* msg)
{
	if(!status)
		return 0;

	if(count>=FIDO_MAX_MSG_COUNT)
		return 0;
		
	UInt16 i;
	
	for(i=0;i<count;i++)
		if(msgid[i]==msg->msgid)
			return 0;
		
	UInt16 pos,datalen;
	
	datalen=GetMessageLen(msg);
	pos=0;
	
	MemHandle mh;
	UInt16 db_index;
	UInt8* memPtr;

	db_index=dmMaxRecordIndex;
	mh=DmNewRecord(dbP,&db_index,datalen);

	if(mh)
	{
		memPtr=(UInt8*)MemHandleLock(mh);

		DmWrite(memPtr,pos,(void*)msg,22);pos+=22;

		DmWrite(memPtr,pos,msg->area,msg->arealen);pos+=msg->arealen;
		DmWrite(memPtr,pos,msg->from,msg->fromlen);pos+=msg->fromlen;
		DmWrite(memPtr,pos,msg->to,msg->tolen);pos+=msg->tolen;
		DmWrite(memPtr,pos,msg->addr,msg->addrlen);pos+=msg->addrlen;
		DmWrite(memPtr,pos,msg->subj,msg->subjlen);pos+=msg->subjlen;
		DmWrite(memPtr,pos,msg->text,msg->textlen);pos+=msg->textlen;

		MemHandleUnlock(mh);
		DmReleaseRecord(dbP,db_index,true);
		
		msgid[count]=msg->msgid;
		reply[count]=msg->reply;
		
		count++;
		unread++;
		
		return 1;
	};
	return 0;
};

char CArea::GetMessageByIndex(SMessage* msg,UInt16 index)
{
	if(!status)
		return 0;
		
	if(index>=count)
		return 0;
		
	UInt16 datalen;
	
	MemHandle mh;
	UInt8* memPtr;

	mh=DmGetRecord(dbP,index+1);

	if(mh)
	{
		memPtr=(UInt8*)MemHandleLock(mh);

		MemMove((void*)msg,memPtr,22);
		datalen=msg->arealen+msg->fromlen+msg->tolen+msg->addrlen+msg->subjlen+msg->textlen;
		
		msg->area=new char [datalen];
		if(!msg->area)
		{
			MemHandleUnlock(mh);
			return 0;
		};
		
		MemMove((void*)msg->area,memPtr+22,datalen);
		msg->from=msg->area+msg->arealen;
		msg->to=msg->area+msg->arealen+msg->fromlen;
		msg->addr=msg->area+msg->arealen+msg->fromlen+msg->tolen;
		msg->subj=msg->area+msg->arealen+msg->fromlen+msg->tolen+msg->addrlen;
		msg->text=msg->area+msg->arealen+msg->fromlen+msg->tolen+msg->addrlen+msg->subjlen;
		
		MemHandleUnlock(mh);
		DmReleaseRecord(dbP,index+1,true);
		
		return 1;
	};
	return 0;
};

UInt16 CArea::GetUnreadCount()
{
	if(status)
		return unread;
	else
		return 0;
};

UInt16 CArea::GetCount()
{
	if(status)
		return count;
	else
		return 0;
};

UInt16 CArea::GetCurrentIndex()
{
	if(status)
		return current_index;
	else
		return 0;
};


char CArea::GetCurrentMessage(SMessage* message)
{
	return GetMessageByIndex(message,current_index);
};

char CArea::DeleteCurrentMessage()
{
	return DeleteMessageByIndex(current_index);
};

char CArea::MoveToFirst()
{
	if(status)
	{
		if(count)
			current_index=0;
		else
			return 0;
			
		return 1;
	}
	else
		return 0;
};

char CArea::MoveToLast()
{
	if(status)
	{
		if(current_index<count-1)
			current_index=count-1;
		else
			return 0;
			
		return 1;
	}
	else
		return 0;
};

char CArea::MoveToFirstUnread()
{
	if(status)
	{
		if(count)
		{
			for(int i=0;i<count;i++)
				if(!GetMessageAttribute(i,FIDO_MSG_ATTR_READ))
				{
					current_index=i;
					return 1;
				}
				
			return 0;
		}
		else
			return 0;
			
		return 1;
	}
	else
		return 0;
};

char CArea::MoveToNextUnread()
{
	if(status)
	{
		if(count)
		{
			for(int i=current_index;i<count;i++)
				if(!GetMessageAttribute(i,FIDO_MSG_ATTR_READ))
				{
					current_index=i;
					return 1;
				}
				
			return 0;
		}
		else
			return 0;
			
		return 1;
	}
	else
		return 0;
};

char CArea::MoveTo(int index)
{
	if(status)
	{
		if(index>=0 && index<count && index!=current_index)
		{
			current_index=index;
			return 1;
		}
		else
			return 0;
	}
	else
		return 0;
};

char CArea::MoveToNext()
{
	if(status)
	{
		if(current_index<count-1)
			current_index++;
		else
			return 0;
			
		return 1;
	}
	else
		return 0;
};

char CArea::MoveToPrev()
{
	if(status)
	{
		if(current_index>0)
			current_index--;
		else
			return 0;

		return 1;
	}
	else
		return 0;

};

char CArea::GetMessageFrom(char* str,int index)
{
	if(index>=0 && index<count)
	{
		SMessage msg;
		
		MemHandle mh;
		UInt8* memPtr;

		mh=DmGetRecord(dbP,index+1);

		if(mh)
		{
			memPtr=(UInt8*)MemHandleLock(mh);
			MemMove((void*)&msg,memPtr,22);
			
			StrCopy(str,(char*)(memPtr+22+msg.arealen));
			
			MemHandleUnlock(mh);
			DmReleaseRecord(dbP,index+1,true);
			return msg.fromlen;
		};
		DmReleaseRecord(dbP,index+1,true);
	}
	return 0;
};

char CArea::MoveToNextReply()
{
	if(status)
	{
		if(count)
		{
			char *item[FIDO_AREA_MAX_REPLY_NUM];
			char buf[FIDO_MAX_NAME_LEN],buflen;
			UInt16 msgindex[FIDO_AREA_MAX_REPLY_NUM],i;
			signed int selected_item_index;
			int itemcount;
			
			itemcount=0;
			
			UInt32 current_msgid;
			
			current_msgid=msgid[current_index];
	
			for(i=0;i<count && itemcount<FIDO_AREA_MAX_REPLY_NUM;i++)
				if(reply[i]==current_msgid)
				{
					msgindex[itemcount]=i;
					
					if(GetMessageFrom(buf,i))
					{
						buflen=StrLen(buf);
						item[itemcount]=new char [buflen+1];
						MemSet(item[itemcount],buflen+1,0);
						StrCopy(item[itemcount],buf);
					}
					else
					{
						item[itemcount]=new char [4];
						MemSet(item[itemcount],32,0);
						StrCopy(item[itemcount],"N\\A");
					};
					itemcount++;
				};
				
			if(!itemcount)
			{
				return 0;
			}
				
			if(itemcount==1)
			{
				current_index=msgindex[0];
				delete[]item[0];
				return 1;
			}
				
			selected_item_index=mlPopUpList(item,itemcount);
			
			if(selected_item_index>=0)
				current_index=msgindex[selected_item_index];
				
			for(i=0;i<itemcount;i++)
				delete[]item[i];
				
			return 1;
		};
	};
	return 0;
};

char CArea::MoveToPrevReply()
{
	if(status)
	{
		if(count)
		{
			UInt16 i;
			
			UInt32 current_reply;
			
			current_reply=reply[current_index];
			
			if(!current_reply)
				return 0;
				
			for(i=0;i<count;i++)
				if(msgid[i]==current_reply)
				{
					current_index=i;
					return 1;
				};
		};
	};
	return 0;
};

void CArea::GenerateTree(char** items,int* itemsindex,int& itemscount,char level,int messageindex)
{
	if(!status)
		return;
		
	if(!(level<FIDO_AREA_MAX_TREE_LEVEL))
		return;
		
	if(!(itemscount<FIDO_AREA_MAX_TREE_LEN))
		return;
		
	int i;
	UInt32 currentmsgid;
	static char buf[FIDO_MAX_NAME_LEN+FIDO_AREA_MAX_TREE_LEN],buflen;
	
	buflen=GetMessageFrom(buf,messageindex);
	if(buflen)
	{
		items[itemscount]=new char [buflen+FIDO_AREA_MAX_TREE_LEN+2];
		itemsindex[itemscount]=messageindex;
//		StrPrintF(items[itemscount],"%4d",messageindex);
		MemSet(items[itemscount],level,'-');

		if(GetMessageAttribute(messageindex,FIDO_MSG_ATTR_READ))
		{
			StrCopy(items[itemscount]+level,buf);
		}
		else
		{
			items[itemscount][level]='[';
			StrCopy(items[itemscount]+level+1,buf);
			StrCat(items[itemscount],"]");
		};
		itemscount++;
	}
		
	currentmsgid=msgid[messageindex];
	for(i=0;i<count;i++)
	{
		if(i==messageindex)
			continue;
				
		if(currentmsgid && reply[i]==currentmsgid)
			GenerateTree(items,itemsindex,itemscount,level+1,i);
	}
	
//	long t=messageindex;
};

char CArea::MoveByTree()
{
	if(!status)
		return 0;
		
	int tmpindex,i;
	UInt32 tmpreply;
	char fn;//,fnlevel;
	
	tmpindex=current_index;
//	fnlevel=0;
		
	do
	{
		fn=0;
		tmpreply=reply[tmpindex];
		
		if(!tmpreply)
			break;
		
		for(i=0;i<count;i++)
		{
			if(i==tmpindex)
				continue;
				
			if(msgid[i]==tmpreply)
			{
				tmpindex=i;
				fn=1;
			}
		}
	}
	while(fn);
	
	char *items[FIDO_AREA_MAX_TREE_LEN];
	int itemsindex[FIDO_AREA_MAX_TREE_LEN];
	int itemscount;
		
	itemscount=0;
	GenerateTree(items,itemsindex,itemscount,0,tmpindex);
	
	fn=0;
	
	if(itemscount>1)
	{
		signed int si;
		
		si=0;
		
		for(i=0;i<itemscount;i++)
			if(current_index==itemsindex[i])
				si=i;
		
		si=mlPopUpList(items,itemscount,si);
		
		if(si>=0)
		{
			current_index=itemsindex[si];
			fn=1;
		};

		for(i=0;i<itemscount;i++)
			delete[]items[i];
	}
	
	return fn;
}

char CArea::MoveByList()
{
	FormType *frmp;
	ListPtr listp;

	int selected_index;
		
	frmp=FrmGetActiveForm();

	if(count<13)
		LstNewList((void**)&frmp,9002,0,0,156,count*13,stdFont,count,0);
	else
		LstNewList((void**)&frmp,9002,0,0,156,11*13,stdFont,13,0);
		
	listp=(ListPtr)FrmGetObjectPtr(frmp,FrmGetObjectIndex(frmp,9002));
	LstSetListChoices(listp,(char**)this,count);

	if(current_index && current_index<count)
	{
		LstSetTopItem(listp,current_index);
		LstSetSelection(listp,current_index);
	}
	
	LstSetDrawFunction(listp,MsgListDrawItem);
	
	selected_index=LstPopupList(listp);
	FrmRemoveObject(&frmp,FrmGetObjectIndex(frmp,9002));
	
	if(selected_index!=current_index && selected_index!=-1)
	{
		current_index=selected_index;
		return 1;
	}
	else
		return 0;
}


char CArea::GetAreaHeader(LocalID dbid,SAreaHeader* sah)
{
	DmOpenRef db;
	
	db=DmOpenDatabase(0,dbid,dmModeReadOnly);
	
	if(!mlCopyRecordToMem(db,0,(void*)sah,0,32))
	{
		FrmCustomAlert(OpenErrorAlert,"  ",NULL,NULL);
		DmCloseDatabase(db);
		return 0;
	};
	
	if(sah->version!=0x2801)
	{
		FrmCustomAlert(OpenErrorAlert,"  ",NULL,NULL);
		DmCloseDatabase(db);
		return 0;
	}
	
	if(sah->fixed!=111)
	{
		sah->delreaddays=0;
		sah->delunreaddays=0;
		sah->attrdelread=0;
		sah->attrdelunread=0;
		sah->attrdobackup=0;
		sah->attrunused=0;
		sah->fixed=111;
	};
	
	DmCloseDatabase(db);
	return 1;
};

char CArea::SetAreaHeader(LocalID dbid,SAreaHeader* sah)
{
	DmOpenRef db;
	
	db=DmOpenDatabase(0,dbid,dmModeReadWrite);
	
	if(!mlWriteToRecord(db,0,(void*)sah,0,32))
	{
		FrmCustomAlert(OpenErrorAlert,"   ",NULL,NULL);
		DmCloseDatabase(db);
		return 0;
	};
	
	DmCloseDatabase(db);
	return 1;
};

char CArea::Purge(LocalID dbid)
{
	if(status)
		return 0;
		
	Open(dbid);
		
	if(!status)
		return 0;
		
	SAreaHeader sah;
	SMessage msg;
	
	mlCopyRecordToMem(dbP,0,(void*)&sah,0,32);
	
	if(sah.fixed!=111)
	{
		sah.delreaddays=0;
		sah.delunreaddays=0;
		sah.attrdelread=0;
		sah.attrdelunread=0;
		sah.attrdobackup=0;
		sah.attrunused=0;
		sah.fixed=111;
	};
	
	if(sah.attrdelread==0 && sah.attrdelunread==0)
	{
		Close();
		return 1;
	}
		
	UInt32 currenttime;
	UInt16 current,total;
	
	
	currenttime=TimGetSeconds();
	
	mlDrawRectangle(10,65,120,10);
	
	total=count;
	current=0;
		
	for(Int16 i=count-1;i>=0;i--)
	{
		mlDrawProgressBar(10,65,120,10,current++,total);

		mlCopyRecordToMem(dbP,i+1,(void*)&msg,0,14);
		
		if(sah.attrdelread && (msg.attributes&FIDO_MSG_ATTR_READ) && ((currenttime-msg.datetime)>(((UInt32)sah.delreaddays)*86400u)))
		{
			DeleteMessageByIndex(i);
			continue;
		};

		if(sah.attrdelunread && (!(msg.attributes&FIDO_MSG_ATTR_READ)) && ((currenttime-msg.datetime)>(((UInt32)sah.delunreaddays)*86400u)))
		{
			DeleteMessageByIndex(i);
			continue;
		};
	}
	Close();
	return 1;
};

UInt16 CArea::Search(char* buf,UInt16 start_index)
{
	if(!status)
		return 0xFFFF;
		
	UInt16 findex;

	mlDrawRectangle(10,130,140,10);
	
	SMessage msg;
	msg.area=NULL;
	
	findex=0xFFFF;

	StrToLower(buf,buf);

	
	for(int i=start_index;i<count;i++)
	{
		mlDrawProgressBar(10,130,140,10,i-start_index,count-start_index);

		if(msg.area)
		{
			delete[]msg.area;
			msg.area=NULL;
		}

		if(GetMessageByIndex(&msg,i))
		{
			StrToLower(msg.area,msg.area);
			if(StrStr(msg.area,buf))
			{
				findex=i;
				i=count;
				break;
			}
			
			StrToLower(msg.from,msg.from);
			if(StrStr(msg.from,buf))
			{
				findex=i;
				i=count;
				break;
			}
			
			StrToLower(msg.to,msg.to);
			if(StrStr(msg.to,buf))
			{
				findex=i;
				i=count;
				break;
			}
			
			StrToLower(msg.subj,msg.subj);
			if(StrStr(msg.subj,buf))
			{
				findex=i;
				i=count;
				break;
			}

			StrToLower(msg.text,msg.text);
			if(StrStr(msg.text,buf))
			{
				findex=i;
				i=count;
				break;
			}
		}
	}
	
	if(msg.area)
	{
		delete[]msg.area;
		msg.area=NULL;
	}
	
	return findex;
};


char CArea::DeleteAllMessages(LocalID dbid,char delete_always)
{
	if(status)
		return 0;
		
	Open(dbid);
		
	if(!status)
		return 0;
		
	SAreaHeader sah;
	
	mlCopyRecordToMem(dbP,0,(void*)&sah,0,32);
	
	if(sah.fixed!=111)
	{
		sah.delreaddays=0;
		sah.delunreaddays=0;
		sah.attrdelread=0;
		sah.attrdelunread=0;
		sah.attrdobackup=0;
		sah.attrunused=0;
		sah.fixed=111;
	};
	
	if(sah.attrcleanenable==0 && delete_always==0)
	{
		Close();
		return 1;
	}
	
	for(Int16 i=count-1;i>=0;i--)
		DeleteMessageByIndex(i);

	Close();
	return 1;	
}


char CArea::GetMsgHeader(int index,char* from_buf,char* to_buf,char* subj_buf,UInt16* attributes)
{
	if(!status)
		return 0;
		
	if(index>=count)
		return 0;
		
	if(!from_buf||!to_buf||!subj_buf)
		return 0;
		
	MemHandle mh;
	UInt8* memPtr;

	mh=DmGetRecord(dbP,index+1);

	if(mh)
	{
		SMessage msg;
		
		memPtr=(UInt8*)MemHandleLock(mh);

		MemMove((void*)&msg,memPtr,22);
		
		StrCopy(from_buf,(const char*)(memPtr+22+msg.arealen));
		StrCopy(to_buf,(const char*)(memPtr+22+msg.arealen+msg.fromlen));
		StrCopy(subj_buf,(const char*)(memPtr+22+msg.arealen+msg.fromlen+msg.tolen+msg.addrlen));
		
		*attributes=msg.attributes;
		
		MemHandleUnlock(mh);
		DmReleaseRecord(dbP,index+1,true);
		
		return 1;
	}
	else
		return 0;
}


void MsgListDrawItem(Int16 itemNum,RectangleType *bounds, Char **itemsText)
{
	CArea *a;
	a=(CArea*)itemsText;
	
	char from[32];
	char to[32];
	char subj[80];
	UInt16 attr;
	
/*	FontID selfont,pf;

	if((FontID)FidoReaderPrefs.CurrentFontID==(FontID)130)
	{
		if(fonth)
			selfont=(FontID)130;
		else
			selfont=stdFont;
	}
	else
		selfont=(FontID)FidoReaderPrefs.CurrentFontID;
		
	pf=FntSetFont(selfont);*/
	
	
	if(a->GetMsgHeader(itemNum,from,to,subj,&attr))
	{
		
		if(!attr&&FIDO_MSG_ATTR_READ)
			WinDrawChars("!",1,bounds->topLeft.x,bounds->topLeft.y);
			
		WinDrawTruncChars(from,StrLen(from),bounds->topLeft.x+4,bounds->topLeft.y,38);
		WinDrawLine(bounds->topLeft.x+41,bounds->topLeft.y,bounds->topLeft.x+41,bounds->topLeft.y+bounds->extent.y);
		WinDrawTruncChars(to,StrLen(to),bounds->topLeft.x+43,bounds->topLeft.y,42);
		WinDrawLine(bounds->topLeft.x+85,bounds->topLeft.y,bounds->topLeft.x+85,bounds->topLeft.y+bounds->extent.y);
		WinDrawTruncChars(subj,StrLen(subj),bounds->topLeft.x+87,bounds->topLeft.y,73);
	}
	
//	FntSetFont(pf);
}


void DoClean()
{
	if(FrmAlert(QCleanAlert)==QCleanOK)
	{
		CArea *area;
		FormType* FrmP;
//		char areaname[35];
	
		LocalID areadbid[FIDO_MAX_AREA_COUNT];
		UInt16 areacount;
		areacount=mlGetDBIDList(areadbid,FIDO_MAX_AREA_COUNT,"fddb","GaUs");
	
		area=new CArea();
	
		FrmP=FrmNewForm(8901," ...",10,50,140,50,true,0,0,0);
		FrmDrawForm(FrmP);
	
		mlDrawRectangle(10,25,120,20);

		for(UInt16 i=0;i<areacount;i++)
		{
//			DmDatabaseInfo(0,areadbid[i],areaname,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			mlDrawProgressBar(10,25,120,20,i,areacount);
		
			area->DeleteAllMessages(areadbid[i]);
		};
	
		delete area;
	
		FrmEraseForm(FrmP);
		FrmDeleteForm(FrmP);
	};
};

void DoPurge()
{
	CArea *area;
	FormType* FrmP;
	RectangleType rect;
	char buf[40],areaname[35];
	
	LocalID areadbid[FIDO_MAX_AREA_COUNT];
	UInt16 areacount;
	areacount=mlGetDBIDList(areadbid,FIDO_MAX_AREA_COUNT,"fddb","GaUs");
	
	area=new CArea();
	
	FrmP=FrmNewForm(8901," ...",10,50,140,80,true,0,0,0);
	FrmDrawForm(FrmP);
//	WinPushDrawState();
	
	mlDrawRectangle(10,35,120,12);

	for(UInt16 i=0;i<areacount;i++)
	{
		DmDatabaseInfo(0,areadbid[i],areaname,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
		
		mlDrawProgressBar(10,35,120,12,i,areacount);
		
		StrPrintF(buf," #%d  %d ",i+1,areacount);
		WinDrawChars(buf,StrLen(buf),10,20);

		RctSetRectangle(&rect,10,50,120,12);
		WinEraseRectangle(&rect,0);
		WinDrawChars(areaname+2,StrLen(areaname+2),10,50);
		
		area->Purge(areadbid[i]);

	};
	
	delete area;
	
//	WinPopDrawState();
	FrmEraseForm(FrmP);
	FrmDeleteForm(FrmP);
};


void DoToss(UInt8 autotoss)
{
	CArea *area;
	FormType* FrmP;
	Spkt pkt;
	DmOpenRef tossdb;
//	RectangleType rect;
	UInt8 tosstype;
	
	LocalID areadbid[FIDO_MAX_AREA_COUNT],tossdbid[FIDO_MAX_TOSS_COUNT];
	char *areaname[FIDO_MAX_AREA_COUNT];
	UInt16 areacount,tosscount,tossi,pktj,i,pktcount,pos,currentindex,findindex,errorcount;
	char buf40[40],buf40len,mtxt[60];

	tosscount=mlGetDBIDList(tossdbid,FIDO_MAX_TOSS_COUNT,"fido","GaUs");
	
	if(tosscount==0)
	{
		if(!autotoss)
			FrmAlert(NothingToTossAlert);
		return;
	};
	
	mlSortDBIDByDBDate(tossdbid,tosscount);

	areacount=mlGetDBIDList(areadbid,FIDO_MAX_AREA_COUNT,"fddb","GaUs");
	
	for(i=0;i<areacount;i++)
	{
		DmDatabaseInfo(0,areadbid[i],buf40,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
		buf40len=StrLen(buf40);
		areaname[i]=new char [buf40len-1];
		MemSet(areaname[i],buf40len-1,0);
		StrCopy(areaname[i],buf40+2);
	};
	
	area=new CArea();
	currentindex=FIDO_MAX_AREA_COUNT+1;
	errorcount=0;

	FrmP=FrmNewForm(8900," ...",10,30,140,100,true,0,0,0);
	FrmDrawForm(FrmP);

//	WinPushDrawState();
	
	if(FidoReaderPrefs.TossType!=FIDO_TOSS_TYPE_AUTO)
		tosstype=FidoReaderPrefs.TossType;

	for(tossi=0;tossi<tosscount;tossi++)
	{
		tossdb=DmOpenDatabase(0,tossdbid[tossi],dmModeReadWrite);
		pktcount=DmNumRecords(tossdb);
		
		StrPrintF(mtxt," #%d  %d ",tossi+1,tosscount);
		WinDrawChars(mtxt,StrLen(mtxt),10,20);
		
		mlDrawRectangle(8,38,124,14);
		
//		RctSetRectangle(&rect,8,38,124,14);
//		WinDrawRectangle(&rect,0);
//		RctSetRectangle(&rect,9,39,122,12);
//		WinEraseRectangle(&rect,0);

		if(FidoReaderPrefs.TossType==FIDO_TOSS_TYPE_AUTO)
			if(mlGetFreeMemorySpace(0)>mlGetDBSize(0,tossdbid[tossi]))
				tosstype=FIDO_TOSS_TYPE_NORMAL;
			else
				tosstype=FIDO_TOSS_TYPE_COMPACT;
	
		for(pktj=0;pktj<pktcount;pktj++)
		{
			mlDrawProgressBar(8,38,124,14,pktj,pktcount);

//			RctSetRectangle(&rect,10,40,(UInt16)((120l*(UInt32)(pktj+1))/(UInt32)pktcount),10);
//			WinDrawRectangle(&rect,0);
		
			StrPrintF(mtxt," #%d  %d      ",pktj+1,pktcount);
			WinDrawChars(mtxt,StrLen(mtxt),10,54);

			MemHandle mh;
			UInt8* memPtr;

			mh=DmGetRecord(tossdb,pktj);

			if(mh)
			{
				memPtr=(UInt8*)MemHandleLock(mh);
				
				pos=4+4+FIDO_MAX_AREA_NAME_LEN+20+FIDO_MAX_NAME_LEN+16+FIDO_MAX_NAME_LEN+FIDO_MAX_SUBJ_LEN+2+2;
				MemMove((void*)&pkt,memPtr,pos);
				pkt.text=(char*)(memPtr+pos);
				
				//  pkt
				
				findindex=FIDO_MAX_AREA_COUNT+2;
				
				for(i=0;i<areacount;i++)
				{
					if(StrCompare(areaname[i],pkt.area)==0)
					{
						findindex=i;
						break;
					};
				};
				
				if(findindex<FIDO_MAX_AREA_COUNT)
				{
					if(findindex!=currentindex)
					{
						area->Close();
						area->Open(areadbid[findindex]);
						currentindex=findindex;
					}
				}
				else
				{
					area->Close();
					buf40len=StrLen(pkt.area);
					areaname[areacount]=new char [buf40len+1];
					MemSet(areaname[areacount],buf40len+1,0);
					StrCopy(areaname[areacount],pkt.area);
					areadbid[areacount]=area->Create(areaname[areacount]);
					currentindex=areacount;
					areacount++;
				};
				
				if(area->AddMessage(&pkt)==0)
					errorcount++;

				MemHandleUnlock(mh);
				DmReleaseRecord(tossdb,pktj,true);
				
				if(tosstype==FIDO_TOSS_TYPE_COMPACT)
					DmDeleteRecord(tossdb,pktj);
				
			};
			
			StrPrintF(mtxt," %d   ",errorcount);
			WinDrawChars(mtxt,StrLen(mtxt),10,68);

		};
		
		area->Close();

		DmCloseDatabase(tossdb);
		DmDeleteDatabase(0,tossdbid[tossi]);
	}

//	WinPopDrawState();

	FrmEraseForm(FrmP);
	FrmDeleteForm(FrmP);

	delete area;
	
	//1  ID   
	//2     ID

	//3      
	//3.1  
	//3.2  
	//3.3      ?
	//3.3.1 . 
	//3.3.2 .   . .
	//4   .
	
	//   .
};









int mlGetDBIDList(LocalID *list,int size,char* type,char* creator)
{
	int items_num;

	DmSearchStateType state;
	Boolean latestVer;
	UInt32 myType,myCreator;
	UInt16 card;
	Err theErr;
	
	latestVer=false;
	LocalID currentDB=0;
	card=0;
	
	myType=*(UInt32*)type;
	myCreator=*(UInt32*)creator;
	
	items_num=0;
	
	theErr=DmGetNextDatabaseByTypeCreator(true,&state, myType, myCreator, latestVer, &card, &currentDB);

	while (!theErr && currentDB && items_num<size)
	{
		list[items_num]=currentDB;
		items_num++;
		theErr=DmGetNextDatabaseByTypeCreator(false,&state,myType,myCreator,latestVer,&card,&currentDB);
	};
	return items_num;
};

void mlSortDBIDByDBDate(LocalID *list,int size)
{
	int i,j,min_index;
	UInt32 min_data,current_data;
	LocalID tmp;
	
	for(i=0;i<size-1;i++)
	{
		min_index=i;
		DmDatabaseInfo(0,list[i],NULL,NULL,NULL,&min_data,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
		for(j=i+1;j<size;j++)
		{
			DmDatabaseInfo(0,list[j],NULL,NULL,NULL,&current_data,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
			if(current_data<min_data)
			{
				min_data=current_data;
				min_index=j;
			};
		};
		if(min_index!=i)
		{
			tmp=list[min_index];
			list[min_index]=list[i];
			list[i]=tmp;
		};
	};
};

char mlCopyRecordToMem(DmOpenRef dbP,UInt16 index,void *data,UInt16 from,UInt16 count)
{
	MemHandle mh;
	UInt8* memPtr;

	mh=DmGetRecord(dbP,index);

	if(mh)
	{
		memPtr=(UInt8*)MemHandleLock(mh);
		MemMove(data,memPtr+from,count);
		MemHandleUnlock(mh);
		DmReleaseRecord(dbP,index,true);
		return 1;
	};
	return 0;
};

char mlAddRecord(DmOpenRef db,void* data,UInt16 datalen)
{
	MemHandle mh;
	UInt16 db_index;
	UInt8* memPtr;

	db_index=dmMaxRecordIndex;
	mh=DmNewRecord(db,&db_index,datalen);

	if(mh)
	{
		memPtr=(UInt8*)MemHandleLock(mh);
		DmWrite(memPtr,0,data,(UInt32)datalen);
		MemHandleUnlock(mh);
		DmReleaseRecord(db,db_index,true);
		return 1;
	};
	return 0;
};

char mlWriteToRecord(DmOpenRef db,UInt16 index,void* data,UInt16 from,UInt16 datalen)
{
	MemHandle mh;
	UInt8* memPtr;

	mh=DmGetRecord(db,index);

	if(mh)
	{
		memPtr=(UInt8*)MemHandleLock(mh);
		DmWrite(memPtr,from,data,datalen);
		MemHandleUnlock(mh);
		DmReleaseRecord(db,index,true);
		return 1;
	};
	return 0;
};

void AreaPopUpListDraw(Int16 itemNum,RectangleType *bounds, Char **itemsText)
{
	char *buf,start,lenght;
	
	buf=itemsText[itemNum];	
	start=0;lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start,lenght,bounds->topLeft.x,bounds->topLeft.y);
	start+=lenght+1;
	lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start-1,1,bounds->topLeft.x+105,bounds->topLeft.y);
	WinDrawChars(buf+start,lenght,bounds->topLeft.x+128-5*lenght,bounds->topLeft.y);

	start+=lenght+1;
	lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start-1,1,bounds->topLeft.x+128,bounds->topLeft.y);
	WinDrawChars(buf+start,lenght,bounds->topLeft.x+150-5*lenght,bounds->topLeft.y);
};

void AreaPopUpListDraw2(Int16 itemNum,RectangleType *bounds, Char **itemsText)
{
	char *buf,start,lenght;
	
	buf=itemsText[itemNum];	
	start=0;lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start,lenght,bounds->topLeft.x,bounds->topLeft.y);
	start+=lenght+1;
	lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start-1,1,bounds->topLeft.x+95,bounds->topLeft.y);
	WinDrawChars(buf+start,lenght,bounds->topLeft.x+118-5*lenght,bounds->topLeft.y);

	start+=lenght+1;
	lenght=0;
	
	while(buf[start+lenght]!='|' && buf[start+lenght])
		lenght++;
	WinDrawChars(buf+start-1,1,bounds->topLeft.x+118,bounds->topLeft.y);
	WinDrawChars(buf+start,lenght,bounds->topLeft.x+138-5*lenght,bounds->topLeft.y);
};

void *GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;
	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

void SetFieldText(UInt16 fieldId,char* text)
{
	static FieldType* fldP;
	fldP=(FieldType*)GetObjectPtr(fieldId);
	FldSetTextPtr(fldP,text);
	FldDrawField(fldP);
};

void SetFieldMLText(UInt16 fieldId,char* text)
{
	static FieldType* fldP;
	fldP=(FieldType*)GetObjectPtr(fieldId);
	FldSetTextPtr(fldP,text);
	FldSetScrollPosition(fldP,0);
	FldRecalculateField (fldP,true);
};


UInt32 mlGetSecondsByStrData(char* s)
{
// 0123456789012345678
//	18 Dec 02  19:26:00   
// 01 Jan 03  15:40:40
	char month[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
	
	DateTimeType dt;
	
	dt.second=(s[17]-'0')*10+(s[18]-'0');
	dt.minute=(s[14]-'0')*10+(s[15]-'0');
	dt.hour=(s[11]-'0')*10+(s[12]-'0');
	
	dt.day=(s[0]-'0')*10+(s[1]-'0');
	
	dt.month=0;
	for(int i=0;i<12;i++)
		if(StrNCompare(month[i],(s+3),3)==0)
			dt.month=i+1;
			
	dt.year=(s[7]-'0')*10+(s[8]-'0');
	
	if(dt.year>70)
		dt.year+=1900;
	else
		dt.year+=2000;
		
	return TimDateTimeToSeconds(&dt);
};

void mlSecondsToStrData(char *str,UInt32 sec)
{
	DateTimeType dt;
	char month[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

	TimSecondsToDateTime(sec,&dt);
	
	StrPrintF(str,"%02d %3s %04d  %02d:%02d\0",dt.day,month[dt.month-1],dt.year,dt.hour,dt.minute);
};

signed int mlPopUpList(char** items,int itemcount,int sselitem)
{
	FormType *frmp;
	ListPtr listp;

	int selected_index;
		
	frmp=FrmGetActiveForm();

	if(itemcount<12)
		LstNewList((void**)&frmp,9001,0,0,155,itemcount*11,stdFont,itemcount,0);
	else
		LstNewList((void**)&frmp,9001,0,0,155,11*12,stdFont,12,0);
		
	listp=(ListPtr)FrmGetObjectPtr(frmp,FrmGetObjectIndex(frmp,9001));
	LstSetListChoices (listp,items,itemcount);

	if(sselitem && sselitem<itemcount)
	{
		LstSetTopItem(listp,sselitem);
		LstSetSelection(listp,sselitem);
	}

	selected_index=LstPopupList(listp);
	FrmRemoveObject(&frmp,FrmGetObjectIndex(frmp,9001));

	return selected_index;
};

FontID mlFontSelect(FontID prevFontID)
{
   FormPtr frmP;
   FontID newFontID;

   newFontID=prevFontID;
	frmP=FrmInitForm(SelFontDlgForm);

	switch(prevFontID)
	{
	case stdFont:
		FrmSetControlGroupSelection(frmP,1,SelFontDlgPbtf0PushButton);
		break;
			
	case boldFont:
		FrmSetControlGroupSelection(frmP,1,SelFontDlgPbtf1PushButton);
		break;
			
	case largeFont:
		FrmSetControlGroupSelection(frmP,1,SelFontDlgPbtf2PushButton);
		break;
			
	case largeBoldFont:
		FrmSetControlGroupSelection(frmP,1,SelFontDlgPbtf3PushButton);
		break;
	};
				
	switch(FrmDoDialog(frmP))
	{
	case 5205:
		switch(FrmGetControlGroupSelection(frmP,1))
		{
		case 1:
			newFontID=stdFont;
			break;
			
		case 2:
			newFontID=boldFont;
			break;
			
		case 3:
			newFontID=largeFont;
			break;
			
		case 4:
			newFontID=largeBoldFont;
			break;
		};
		break;
	case SelFontDlgBtCustomButton:
		{
			LocalID fontlist[32];
			char fontcount;
			fontcount=mlGetDBIDList(fontlist,32,"Font","Font");
			
			if(fontcount)
			{
				mlSortDBIDByDBDate(fontlist,fontcount);
				char *fontnamelist[32];
				int sel;
			
				for(char i=0;i<fontcount;i++)
				{
					fontnamelist[i]=new char [35];
					DmDatabaseInfo(0,fontlist[i],fontnamelist[i],NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
				}
					
				sel=mlPopUpList(fontnamelist,fontcount);
				
				if(sel>=0)
				{
					newFontID=(FontID)130;
					FidoReaderPrefs.fontdbid=fontlist[sel];
				}
				
				for(char i=0;i<fontcount;i++)
					delete[]fontnamelist[i];
			}
		}
		break;
	};

	FrmDeleteForm(frmP);
	return newFontID;
};

UInt32 mlGetFreeMemorySpace(UInt16 cardNo)
{
	UInt32 r;
	
	if(MemCardInfo(cardNo,NULL,NULL,NULL,NULL,NULL,NULL,&r)==errNone)
		return r;
	else
		return 0;
};

UInt32 mlGetDBSize(UInt16 cardNo,LocalID dbID)
{
	UInt32 r;
	if(DmDatabaseSize(cardNo,dbID,NULL,&r,NULL)==errNone)
		return r;
	else
		return 0;
};

char mlSetFieldValue(FieldType* fldP,UInt16 value)
{
	MemHandle textH;
	MemPtr textP;

	textH=FldGetTextHandle(fldP);
	
	MemHandleResize(textH,4);
		
	textP=MemHandleLock(textH);
	
	if(textP)
		StrPrintF((char*)textP,"%d",value);
	else
	{
		FldSetTextHandle(fldP,NULL);
		return 0;
	}
		
	MemHandleUnlock(textH);
	FldSetTextHandle(fldP,textH);
//	FldDrawField(fldP);
	
	return 1;
}

char mlGetFieldValue(FieldType* fldP,UInt16& value)
{
	MemHandle textH;
	MemPtr textP;

	textH=FldGetTextHandle(fldP);
	textP=MemHandleLock(textH);
	
	if(textP)
		value=StrAToI((char*)textP);
	else
		return 0;
		
	MemHandleUnlock(textH);
	
	return 1;
}

void mlDrawRectangle(int left,int top,int width,int height)
{
	static RectangleType rect;
	RctSetRectangle(&rect,left,top,width,height);
	WinDrawRectangle(&rect,0);
	RctSetRectangle(&rect,left+1,top+1,width-2,height-2);
	WinEraseRectangle(&rect,0);
}

void mlEraseRectangle(int left,int top,int width,int height)
{
	static RectangleType rect;
	RctSetRectangle(&rect,left,top,width,height);
	WinEraseRectangle(&rect,0);
}

char mlDrawBitmap(int x,int y,UInt32 id)
{
	BitmapPtr bp;
	MemHandle mhbp;
	
	mhbp=DmGetResource(*(DmResType*)"Tbmp",(DmResID)id);
	if(mhbp)
	{
		bp=(BitmapPtr)MemHandleLock(mhbp);
		if(bp)
		{
			WinDrawBitmap(bp,x,y);
			MemHandleUnlock(mhbp);
			DmReleaseResource(mhbp);
			return 1;
		}
		else
		{
			DmReleaseResource(mhbp);
			return 0;
		}
	}
	return 0;
}

void mlDrawProgressBar(int left,int top,int width,int height,int current,int total)
{
	static RectangleType rect;
	static UInt16 pos;
	
	left+=2;
	top+=2;
	width-=4;
	height-=4;
	
	pos=(UInt16)(((UInt32)width*(UInt32)(current+1))/(UInt32)total);

	RctSetRectangle(&rect,left,top,pos,height);
	WinDrawRectangle(&rect,0);
	RctSetRectangle(&rect,left+pos,top,width-pos,height);
	WinEraseRectangle(&rect,0);
};

void mlSetDBBackUpAttribute(LocalID dbid,UInt8 backupattribute)
{
	static UInt16 attributes;
	
	DmDatabaseInfo(0,dbid,NULL,&attributes,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	
	if(backupattribute)
		attributes|=dmHdrAttrBackup;
	else
		attributes&=!dmHdrAttrBackup;
	
	DmSetDatabaseInfo(0,dbid,NULL,&attributes,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
};

char mlGetString(char* buf,int maxlen,char* message,char* caption,char usebuf)
{
	int rez;
	FormType* FrmP;
	FormLabelType* FrmL;
	FieldType* FrmF;
	ControlType *FrmC_OK,*FrmC_Cancel;
	MemHandle mh;
	char *textP;
		
	FrmP=FrmNewForm(8900,caption,20,40,120,56,true,0,0,0);
	FrmL=FrmNewLabel(&FrmP,8901,message,4,12,stdFont);
	FrmF=FldNewField((void**)&FrmP,8902,4,24,112,12,stdFont,maxlen,true,true,true,false,leftAlign,true,false,false);
	
	mh=MemHandleNew(maxlen);
	textP=(char*)MemHandleLock(mh);
	MemSet(textP,maxlen,0);
	if(usebuf)
		MemMove(textP,buf,StrLen(buf));
	FldSetTextHandle(FrmF,mh);
	MemHandleUnlock(mh);

	FrmC_OK=CtlNewControl((void **)&FrmP,8903,buttonCtl,"OK",15,40,35,12,stdFont,0,false);
	FrmC_Cancel=CtlNewControl((void **)&FrmP,8904,buttonCtl,"Cancel",70,40,35,12,stdFont,0,false);
	
	FrmSetFocus(FrmP,FrmGetObjectIndex(FrmP,8902));
	
	rez=FrmDoDialog(FrmP);

	if(rez==8903)
	{
		StrCopy(buf,textP);
		rez=1;
	}
	else
		rez=0;
	
	FrmDeleteForm(FrmP);

	return rez;
};

extern UInt32 curromVersion;
#define palmos35Version	sysMakeROMVersion(3,5,0,sysROMStageRelease,0)

char mlIsDBPresent(LocalID dbid)
{
	if(!dbid)
		return 0;
		
	if(curromVersion<palmos35Version)
	{
		UInt16 index,max_index;
	
		max_index=DmNumDatabases(0);
	
		for(index=0;index<max_index;index++)
			if(DmGetDatabase(0,index)==dbid)
				return 1;

		return 0;
	}
	else
	{	
		if(DmDatabaseProtect(0,dbid,true)==errNone)
		{
			DmDatabaseProtect(0,dbid,false);
			return 1;
		}
		else
			return 0;
	}
}

char mlGetGoToCommand(UInt16 totalmsgcount,UInt16 &currentmsgnumber)
{
   FormPtr frmP;
   FieldType* fldP;
   char return_command;
   char buf[16];

	frmP=FrmInitForm(GotomsgForm);
	
	FrmSetFocus(frmP,FrmGetObjectIndex(frmP,gotomsgMsgNumberField));
	
	fldP=(FieldType*)FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,gotomsgAreaStatFieldField));
	StrPrintF(buf,": %d",totalmsgcount);
	FldSetTextPtr(fldP,buf);
			
	fldP=(FieldType*)FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP,gotomsgMsgNumberField));
	
	MemHandle mh;
	char *textP;

	mh=MemHandleNew(8);
	textP=(char*)MemHandleLock(mh);
	MemSet(textP,8,0);
	StrPrintF(textP,"%d",currentmsgnumber+1);
	FldSetTextHandle(fldP,mh);
	MemHandleUnlock(mh);
	
	return_command=GOTO_CANCEL;
	
				
	switch(FrmDoDialog(frmP))
	{
	case gotomsgBtOKButton:
		return_command=GOTO_YES;
		
		mh=FldGetTextHandle(fldP);
		textP=(char*)MemHandleLock(mh);
	
		if(textP)
			currentmsgnumber=StrAToI(textP);
		else
			currentmsgnumber=0;
		
		MemHandleUnlock(mh);

		if(currentmsgnumber>totalmsgcount || currentmsgnumber<1)
			return_command=GOTO_CANCEL;
		else
			currentmsgnumber--;

		break;

	case gotomsgBtCancelButton:
		return_command=GOTO_CANCEL;
		break;

	case gotomsgGFirstButton:
		return_command=GOTO_FIRST;
		break;

	case gotomsgGLastButton:
		return_command=GOTO_LAST;
		break;

	case gotomsgGFirstUnreadButton:
		return_command=GOTO_FIRST_UNREAD;
		break;

	case gotomsgGNextUnreadButton:
		return_command=GOTO_NEXT_UNREAD;
		break;
	};

	FrmDeleteForm(frmP);
	
	return return_command;
};

char mlNewRecordInMemoPad(char* text)
{
	DmOpenRef				memodb;
	MemHandle				rec;
	UInt16					recNum;
	UInt16					size;
	DmSearchStateType		searchState;
	UInt16					memoDBCard;
	LocalID					memoDBID;

	size=StrLen(text);
	if(size>MEMOPAD_MAX_TEXT_SIZE)
	{
   	size=MEMOPAD_MAX_TEXT_SIZE+1;
		text[MEMOPAD_MAX_TEXT_SIZE]=0;
	}

	if((DmGetNextDatabaseByTypeCreator(true,&searchState,*(UInt32*)("DATA"),*(UInt32*)("memo"),true,&memoDBCard,&memoDBID)!=errNone) || (memodb=DmOpenDatabase(memoDBCard,memoDBID,dmModeWrite))==0)
		return 0;

	recNum=dmMaxRecordIndex;
	rec=DmNewRecord(memodb,&recNum,size+1);
	if(rec==0)
	{
		DmCloseDatabase(memodb);
		return 0;
	}

	DmWrite(MemHandleLock(rec),0,text,size+1);

	MemHandleUnlock(rec);
	DmReleaseRecord(memodb,recNum,true);
	DmCloseDatabase(memodb);
	
	return 1;
};

char mlGetSearchString(char* buf)
{
   FormPtr frmP;
   FieldType* fldP;
   char return_command;

	frmP=FrmInitForm(SearchDlgForm);
	
	FrmSetFocus(frmP,FrmGetObjectIndex(frmP,SearchDlgSearchStringField));
			
	fldP=(FieldType*)FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,SearchDlgSearchStringField));
	
	MemHandle mh;
	char *textP;

	mh=MemHandleNew(SEARCH_MAX_STR_LEN);
	textP=(char*)MemHandleLock(mh);
	MemSet(textP,SEARCH_MAX_STR_LEN,0);
	StrCopy(textP,buf);
	FldSetTextHandle(fldP,mh);
	MemHandleUnlock(mh);
	
	FldSetSelection(fldP,0,StrLen(buf));
	
	return_command=SEARCH_CANCEL;
				
	switch(FrmDoDialog(frmP))
	{
	case SearchDlgStartButton:
		return_command=SEARCH_START;
		break;

	case SearchDlgContinueButton:
		return_command=SEARCH_CONTINUE;
		break;

	case SearchDlgCancelButton:
		return_command=SEARCH_CANCEL;
		break;
	};
	
	if(return_command!=SEARCH_CANCEL)
	{
		mh=FldGetTextHandle(fldP);
		textP=(char*)MemHandleLock(mh);
	
		if(textP)
			StrCopy(buf,textP);
		
		MemHandleUnlock(mh);
	}
	
	mh=FldGetTextHandle(fldP);
	FldSetTextHandle(fldP,NULL);
	
	MemHandleFree(mh);

	FrmDeleteForm(frmP);
	
	return return_command;
};

void mlSetFieldText(UInt16 fieldID,char *text)
{
	MemHandle oldTxtHandle;
	MemHandle txtHandle;
	FieldPtr field;

	txtHandle=MemHandleNew(StrLen(text)+1);
	StrCopy((char*)MemHandleLock(txtHandle),text);
	MemHandleUnlock(txtHandle);
	
	field=(FieldPtr)GetObjectPtr(fieldID);

	oldTxtHandle=FldGetTextHandle(field);

	FldSetTextHandle(field,txtHandle);
	if(oldTxtHandle)
		MemHandleFree(oldTxtHandle);
}


CMessageViewer::CMessageViewer()
{
	status=0;
	linecount=0;
	topline=0;
	linesperscreen=0;
	left=0;top=0;
	width=160;height=160;
	interval=0;
	
	for(int i=0;i<CMV_MAX_LINE_COUNT;i++)
	{
		line[i]=NULL;
		linelenght[i]=0;
	}
}

char CMessageViewer::addline(char* buf,UInt8 len)
{
	if(status)
	{
		line[linecount]=new char [len+1];
		linelenght[linecount]=len;
		StrCopy(line[linecount],buf);
		linecount++;
		return 1;
	}
	else
		return 0;
}

char mlCharIsDelimiter(char c)
{
	switch(c)
	{
	case ' ':
	case '-':
	case '!':
	case '@':
	case '_':
	case '=':
	case '+':
	case '/':
	case '\\':
	case '?':
	case '.':
	case ',':

/*	case '[':
	case ']':
	case '(':
	case ')':
	case '{':
	case '}':*/

	
		return 1;
	default:
		return 0;
	}
}


char CMessageViewer::Init(char* text,int _left,int _top,int _width,int _height,FontID _font)
{
	if(status)
		return 0;
		
	if(!text)
		return 0;
		
	if(StrLen(text)>32760)
		return 0;
		
	font=_font;
		
	FontID prevfont;
	prevfont=FntSetFont(font);
		
	status=1;
		
	left=_left;top=_top;
	width=_width;height=_height;
	
	lineheight=FntCharHeight();
	linesperscreen=(height/lineheight);
	
	linecount=0;
	topline=0;
	
	char linebuf[512],linebuflen;
	MemSet(linebuf,512,0);linebuflen=0;
	
	char *quote,quotelen,currentcharwidth;
	quotelen=0;
	
	char *lastdelimiter;
	lastdelimiter=text;
	quote=text;
	
	int pixelpos,linepos,srclinepos,quotepixlen;
	
	pixelpos=0;
	srclinepos=0;
	linepos=0;
	quotepixlen=0;
	
	while(*text)
	{
		currentcharwidth=FntCharWidth(*text);
		
		if(pixelpos+currentcharwidth>width)
		{	// 
			if(FntCharsWidth(lastdelimiter,text-lastdelimiter)>=(width-currentcharwidth-quotepixlen))
				lastdelimiter=text;
		
			linepos-=text-lastdelimiter;
			text=lastdelimiter;
			
			linebuf[linepos]=0;
			addline(linebuf,linepos);
			
			MemMove(linebuf,quote,quotelen);
			linepos=quotelen;
			pixelpos=quotepixlen;
		}
		else
		{
			if(*text==0x0A)
			{
				linebuf[linepos]=0;
				addline(linebuf,linepos);

				pixelpos=0;
				linepos=0;
				srclinepos=0;
				quotelen=0;
				quotepixlen=0;
				text++;
				quote=text;
			}
			else
			{
				if(mlCharIsDelimiter(*text))
					lastdelimiter=text+1;
				
				if(*text=='>' && srclinepos<16)
				{
					quotelen=srclinepos+1;
					quotepixlen=pixelpos+currentcharwidth;
				}
				
				linebuf[linepos]=*text;
				linepos++;
				srclinepos++;
				pixelpos+=currentcharwidth;
				
				text++;
			}
		}
	}
	
	if(linepos)
	{
		linebuf[linepos]=0;
		addline(linebuf,linepos);
	}
	
	topline=0;
	
	FntSetFont(prevfont);
	
	return status;
}

char CMessageViewer::Free()
{
	if(status)
	{
		for(int i=0;i<linecount;i++)
		{
			if(line[i])
			{
				delete[]line[i];
				line[i]=NULL;
			}
			linelenght[i]=0;
		}
		status=0;
		linecount=0;
		topline=0;
		linesperscreen=0;
		return 1;
	}
	else
		return 0;
}


char CMessageViewer::Show()
{
	if(status)
	{
		FontID prevfont;
		prevfont=FntSetFont(font);
		
		mlEraseRectangle(left,top,width,height);
		for(int i=0;i<linesperscreen && topline+i<linecount;i++)
			WinDrawChars(line[topline+i],linelenght[topline+i],left,top+i*lineheight);
			
		if(linecount>linesperscreen)
		{
			WinDrawGrayLine(left+width,top,left+width,top+height);
		
			if(((Int32)height*(Int32)linesperscreen/linecount+(Int32)height*(Int32)topline/linecount)<height)
				WinDrawLine(left+width,top+(Int32)height*(Int32)topline/linecount,left+width,top+(Int32)height*(Int32)linesperscreen/linecount+(Int32)height*(Int32)topline/linecount);
			else
				WinDrawLine(left+width,top+height-(Int32)height*(Int32)linesperscreen/linecount,left+width,top+height);
		}
		else
			WinDrawLine(left+width,top,left+width,top+height);
		
		FntSetFont(prevfont);
		
		return true;
	}
	else
		return false;

}

char CMessageViewer::LineDown()
{
	if(status)
	{
		if(topline-linesperscreen+1<linecount)
		{
			topline++;
			
			return true;
		}
		else
			return false;
	}
	else
		return false;
}

char CMessageViewer::LineUp()
{
	if(status)
	{
		if(topline-1>0)
			topline--;
		else
			topline=0;
		return true;
	}
	else
		return false;
}

char CMessageViewer::PageDown()
{
	if(status)
	{
		if(topline+linesperscreen<linecount)
		{
			topline+=linesperscreen;
			
			return true;
		}
		else
			return false;
	}
	else
		return false;
}

char CMessageViewer::PageUp()
{
	if(status)
	{
		if(topline-linesperscreen>0)
		{
			topline-=linesperscreen;
			return true;
		}
		else
		{
			if(topline>0)
			{
				topline=0;
				return true;
			}
		}
		
		return false;
	}
	else
		return false;
}


char CMessageViewer::ScrollByBar(int y)
{
	if(y>top && y<top+height && linecount>linesperscreen)
	{
		topline=(Int32)(y-top)*(Int32)linecount/height;
		
		if(topline+linesperscreen>linecount)
			topline=linecount-linesperscreen;
	
		return 1;
	}
	else
		return 0;
}
