#include <PalmOS.h>

#define FIDO_TOSS_DB_VER			0x02

#define FIDO_MAX_DATA_LEN			20
#define FIDO_MAX_AREA_COUNT		128
#define FIDO_MAX_TOSS_COUNT		32
#define FIDO_MAX_AREA_NAME_LEN	40
#define FIDO_MAX_NAME_LEN			36
#define FIDO_MAX_MSG_COUNT			1024
#define FIDO_MAX_SUBJ_LEN			72

#define FIDO_MSG_ATTR_READ			0x0001
#define FIDO_MSG_ATTR_SELECTED	0x0002
#define FIDO_MSG_ATTR_NEW			0x0004
#define FIDO_MSG_ATTR_DELETED		0x0008

#define FIDO_AREA_MAX_REPLY_NUM	128
#define FIDO_AREA_MAX_TREE_LEN	256
#define FIDO_AREA_MAX_TREE_LEVEL	16

#define FIDO_TOSS_TYPE_NORMAL		0
#define FIDO_TOSS_TYPE_COMPACT	1
#define FIDO_TOSS_TYPE_AUTO		2

#define SECONDS_IN_DAY				86400u

#define CMV_MAX_LINE_COUNT			2048

char mlAddRecord(DmOpenRef db,void* data,UInt16 datalen);
int mlGetDBIDList(LocalID *list,int size,char* type,char* creator);
void mlSortDBIDByDBDate(LocalID *list,int size);
char mlCopyRecordToMem(DmOpenRef dbP,UInt16 index,void *data,UInt16 from,UInt16 count);
char mlWriteToRecord(DmOpenRef db,UInt16 index,void* data,UInt16 from,UInt16 datalen);
UInt32 mlGetSecondsByStrData(char* s);
void mlSecondsToStrData(char *str,UInt32 sec);
//int mlGetString(char* buf,int maxlen,char* message,char* caption,char usebuf);
LocalID mlGetNameAndCreateDB(char* type,char* creator);
signed int mlPopUpList(char** items,int itemcount,int sselitem=0);
FontID mlFontSelect(FontID prevFontID);
UInt32 mlGetFreeMemorySpace(UInt16 cardNo);
UInt32 mlGetDBSize(UInt16 cardNo,LocalID dbID);
char mlSetFieldValue(FieldType* fldP,UInt16 value);
char mlGetFieldValue(FieldType* fldP,UInt16& value);
void mlDrawRectangle(int left,int top,int width,int height);
void mlEraseRectangle(int left,int top,int width,int height);
void mlDrawProgressBar(int left,int top,int width,int height,int current,int total);
void mlSetDBBackUpAttribute(LocalID dbid,UInt8 backupattribute);
char mlGetString(char* buf,int maxlen,char* message,char* caption,char usebuf);
char mlIsDBPresent(LocalID dbid);

void mlSetFieldText(UInt16 fieldID,char *text);

char mlCharIsDelimiter(char c);
char mlDrawBitmap(int x,int y,UInt32 id);

//#define ALLOW_ACCESS_TO_INTERNALS_OF_LISTS

////////////////////////////////////////////////////////////////////////
// ,   
//  - ,  
#define SEARCH_START				0x20
#define SEARCH_CONTINUE			0x21
#define SEARCH_CANCEL			0x0
#define SEARCH_MAX_STR_LEN		64
char mlGetSearchString(char* buf);
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
// ,        
//  - ,  
#define GOTO_FIRST				0x10
#define GOTO_LAST					0x11
#define GOTO_FIRST_UNREAD		0x12
#define GOTO_NEXT_UNREAD		0x13
#define GOTO_YES					0x14
#define GOTO_CANCEL				0x0
char mlGetGoToCommand(UInt16 totalmsgcount,UInt16 &currentmsgnumber);
////////////////////////////////////////////////////////////////////////

#define MEMOPAD_MAX_TEXT_SIZE	3900
char mlNewRecordInMemoPad(char* text);

void MsgListDrawItem(Int16 itemNum,RectangleType *bounds, Char **itemsText);

void AreaPopUpListDraw(Int16 itemNum,RectangleType *bounds, Char **itemsText);
void AreaPopUpListDraw2(Int16 itemNum,RectangleType *bounds, Char **itemsText);
void *GetObjectPtr(UInt16 objectID);
void SetFieldText(UInt16 fieldId,char* text);
void SetFieldMLText(UInt16 fieldId,char* text);

void DoToss(UInt8 autotoss); //
void DoPurge(); //
void DoClean(); //

class CFidoDBList;
class CArea;
class CMSG;

#pragma pack(1)
struct SPreferenceType
{
	UInt8 replaceme;
	UInt8 TossType;
	UInt8 SelectedArea;
	UInt8 CurrentFontID;
	UInt8 CardNum;
	UInt8 attrAutoRunTosser:1;
	UInt8 attrAutoRunPurger:1;
	UInt8 attrUnUsed:6;
	UInt32 fontdbid;
	UInt8 reserved[26];
};
#pragma pack()

struct SAreaHeader
{
	UInt16 version;
	UInt16 count;
	UInt16 unread;
	UInt16 current_index;
	
	UInt16 delreaddays;
	UInt16 delunreaddays;

	UInt8 attrdelread:1;
	UInt8 attrdelunread:1;
	UInt8 attrdobackup:1;
	UInt8 attrcleanenable:1;
	UInt8 attrunused:4;

	UInt8 reserved[18];
	UInt8 fixed;
};

struct SMessage
{
	UInt32 msgid,reply;
	UInt16 attributes;
	UInt32 datetime;
	char arealen,fromlen,tolen,addrlen,subjlen;
	UInt16 textlen;
	char *area,*from,*to,*addr,*subj,*text;
};

struct Spkt
{
	UInt32 msgid,reply;
	char area[FIDO_MAX_AREA_NAME_LEN];
	char data[FIDO_MAX_DATA_LEN];
	char from[FIDO_MAX_NAME_LEN];
	char addr[16];
	char to[FIDO_MAX_NAME_LEN];
	char subj[FIDO_MAX_SUBJ_LEN];
	UInt16 attr;
	UInt16 textlen;
	char *text;
};

class CFidoDBList
{
private:
	char status;
	char *name[FIDO_MAX_AREA_COUNT];
	LocalID dbid[FIDO_MAX_AREA_COUNT];
	int area_count;
	UInt16 all_area_size;
	
	void Swap(int i1,int i2);
		
public:
//	int selected_item_index;
	CFidoDBList();
	~CFidoDBList();
	void Free();
	void Init();
	void Sort();
	
	LocalID GetLocalID(UInt16 index);
	LocalID SelectDB();
	void ToList(ListType* lstP);
	UInt16 GetAreaCount()
	{
		return area_count;
	};

	UInt16 GetAllAreaSize()
	{
		return all_area_size;
	};
};

class CArea
{
//     
//   
//      +    ( )
// public:
// 1. get current        
// 2. move next|prev
// 3. delete msg
// 4. add msg
// 5. delete current

//   
// 1. open area(     bool whith_scan)
// 2. close area    

// c  
// 1. delete all message
// 2. sort by date (   )

private:
	DmOpenRef dbP;
	char status;
//	char *name;
	UInt16 count,unread,current_index;

	UInt32 msgid[FIDO_MAX_MSG_COUNT];
	UInt32 reply[FIDO_MAX_MSG_COUNT];

	void SetMessageAttributes(int index,UInt16 attr); //fix
	UInt16 GetMessageAttributes(int index); //fix
	UInt16 GetMessageLen(SMessage* message); //fix
	
	char GetMessageFrom(char* str,int index);

	void Free(); //fix
	
	char GetMessageByIndex(SMessage* mmessage,UInt16 index); //fix
	char DeleteMessageByIndex(UInt16 index); //fix
	
	void GenerateTree(char** items,int* itemsindex,int& itemscount,char level,int messageindex);
	
public:
	CArea(); //fix
	~CArea(); //fix
	
	LocalID Create(char *area_name); //fix
	char Open(LocalID dbid); //fix
	char Close(); //fix
	char Purge(LocalID dbid);
	char DeleteAllMessages(LocalID dbid,char delete_always=0);
	
	static char GetAreaHeader(LocalID dbid,SAreaHeader* sah);
	static char SetAreaHeader(LocalID dbid,SAreaHeader* sah);

	char AddMessage(Spkt* pkt); //fix
	char AddMessage(SMessage* msg); //fix

	char GetCurrentMessage(SMessage* message); //fix
	char DeleteCurrentMessage(); //fix
	
	char MoveToNext(); //fix
	char MoveToPrev(); //fix
	
	char MoveTo(int index);
	
	char MoveToNextReply();
	char MoveToPrevReply();
	
	char MoveToFirst();
	char MoveToFirstUnread();
	char MoveToNextUnread();
	char MoveToLast();
	
	char MoveByTree();
	char MoveByList();
	
	UInt16 GetUnreadCount(); //fix
	UInt16 GetCount(); //fix
	UInt16 GetCurrentIndex(); //fix
	
	void SetMessageAttribute(int index,char value,UInt16 msg_attr); //fix
	char GetMessageAttribute(int index,UInt16 msg_attr); //fix
	
	UInt16 Search(char* buf,UInt16 start_index=0);
	
	char GetMsgHeader(int index,char* from_buf,char* to_buf,char* subj_buf,UInt16* attributes);


//	void SetUnreadCount(int i);
};

/*
class CMsgListView
{
private:

	struct msgh
	{
		char from[32];
		char to[32];
		char subj[80];
	};
	
	msgh line[16];
	
	int top,selected;
	
	int x,y,width,height;


	
};
*/



class CMessageViewer
{
private:
	char status;
	
	char *line[CMV_MAX_LINE_COUNT];
	UInt8 linelenght[CMV_MAX_LINE_COUNT];
	int linecount,topline,linesperscreen,lineheight;
	
	int left,top,width,height;
	char interval;
	
	FontID font;
	
	char addline(char* buf,UInt8 len);

public:

	CMessageViewer();

	char Init(char* text,int _left,int _top,int _width,int _height,FontID _font);
	char Free();
	
	char Show();
	
	char LineDown();
	char LineUp();
	
	char PageDown();
	char PageUp();
	
	char ScrollByBar(int y);

};


